/**
 Implements the citeref field default user interface.

 To make the AJAX call less sensitive and only trigger after the user has finished typing,
 This script involves listening for the keyup event and delaying the AJAX call
 until a certain amount of time has passed without any further keystrokes.
 */
(function ($, Drupal, drupalSettings) {
  Drupal.behaviors.debouncedAjax = {
    attach: function (context, settings) {
      // Timer identifier
      var typingTimer;
      // Time in ms to wait for the user to stop typing
      var doneTypingInterval = 1000;

      // Listen for the keyup event on the text field
      $(once('debouncedAjax', $(".debounce-ajax", context))).on("keyup", function () {
        // Clear the timer if it exists
        clearTimeout(typingTimer);
        typingTimer = setTimeout(function () {
          // Trigger the AJAX call after the user stops typing
          $(this).trigger('debouncedInput');
        }.bind(this), doneTypingInterval);
      });

      // Manually trigger the AJAX call when the debouncedInput event is fired
      $(once('debouncedAjaxTrigger', $(".debounce-ajax", context))).on("debouncedInput", function () {
        // Retrieve the wrapper ID from the data attribute
        var wrapper = $(this).data('wrapper');
        // Check if Drupal.ajax object is available and contains our element's ID
        if (typeof Drupal.ajax !== 'undefined' && typeof Drupal.ajax[this.id] !== 'undefined') {
          // Manually trigger the AJAX request
          Drupal.ajax[this.id].doRequest();
        } else {
          console.error('AJAX object not found for element:', this.id);
        }
      });
    },
  };
}(jQuery, Drupal, drupalSettings));
