<?php

namespace Drupal\citeref_field\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Citation/Reference Controller.
 *
 * Fetches data through the list of citation style languages locally
 * and autocomplete it.
 */
class CiterefFieldController extends ControllerBase {
  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  public function __construct(ModuleHandlerInterface $moduleHandler) {
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
          $container->get('module_handler')
      );
  }

  /**
   * Fetches the specified number of the citation styles from local file.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   return the json format for the retrieved results
   */
  public function handleAutocomplete(Request $request): JsonResponse {
    $results = [];
    $input = $request->query->get('q');

    if (!$input) {
      return new JsonResponse($results);
    }
    $input = Xss::filter($input);
    $needle = trim(strtolower($input));

    if (strlen($needle) <= 2) {
      return new JsonResponse($results);
    }
    $module_path = $this->moduleHandler->getModule('citeref_field')->getPath();
    $csl_styles_file = $module_path . '/csl_styles/csl_styles.txt';
    $csl_style_names = explode("\n", file_get_contents($csl_styles_file));

    foreach ($csl_style_names as $style_name) {
      if (stripos($style_name, $needle) !== FALSE) {
        $results[] = [
          'value' => $style_name,
          'label' => $style_name,
        ];
      }
    }

    return new JsonResponse($results);
  }

}
