<?php

namespace Drupal\citeref_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;

/**
 * Plugin implementation of the 'Default' formatter.
 *
 * @FieldFormatter(
 *   id = "citeref_field_default",
 *   label = @Translation("Default"),
 *   field_types = {
 *     "citeref_field"
 *   }
 * )
 */
class CiterefFieldDefaultFormatter extends FormatterBase {

  /**
   * Plugin implementation of the 'Default' formatter.
   *
   * The default formatter configures simply the citation identifier link,
   * citation type, citation style, and formatted citation.
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {
      $values = $item->getValue();
      $element[$delta] = [
        '#theme' => 'citeref_field_default',
        '#attributes' => [
          'class' => [
            'citeref_field',
          ],
        ],
      ];
      $element[$delta]['#citeref_type'] = $values['citeref_type'];
      $element[$delta]['#citeref_style'] = $values['citeref_style'];
      $element[$delta]['#citeref_id'] = $values['citeref_id'];
      if (isset($values['citeref_record']) && trim($values['citeref_record']) !== '') {
        // Explode the string by line breaks.
        $lines = explode("\n", $values['citeref_record']);
        // Start HTML output.
        $renderHtml = '';

        if ($values['citeref_type'] === 'ARK') {
          foreach ($lines as $line) {
            if (trim($line) !== '') {
              // Split each line into key-value pairs.
              $parts = explode(':', $line, 2);
              if (count($parts) === 2) {
                $key = trim($parts[0]);
                $value = trim($parts[1]);
                $renderHtml .= '<strong>' . $key . ':</strong> ' . $value . '<br>';
              }
            }
          }
        }
        else {
          foreach ($lines as $line) {
            if (trim($line) !== '') {
              $renderHtml .= $line . '<br>';
            }
          }
        }
        $values['citeref_record'] = $renderHtml;
        // Regular expression pattern to find URLs with both http and https,
        // excluding certain punctuation at the end.
        $url_pattern = '/(https?:\/\/[^\s"\'()<>\[\]]+[^.,;"\':()<>\[\]\s])/';

        // Replace URLs with anchor tags.
        $text_with_links = preg_replace($url_pattern, '<a href="$1" target="_blank">$1</a>', $values['citeref_record']);
      }
      else {
        $text_with_links = NULL;
      }
      $element[$delta]['#citeref_record'] = $text_with_links;
    }

    return $element;
  }

}
