<?php

namespace Drupal\citeref_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Defines the 'citeref_field' field type.
 *
 * @FieldType(
 *   id = "citeref_field",
 *   label = @Translation("Citeref Field"),
 *   category = @Translation("General"),
 *   default_widget = "citeref_field_widget",
 *   default_formatter = "citeref_field_default"
 * )
 */
class CiterefFieldItem extends FieldItemBase {

  /**
   * The method determines when the field is empty.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function isEmpty() {
    $value = $this->get('citeref_id')->getValue();
    if ($value === NULL || $value === '') {
      $value = $this->get('citeref_record')->getValue();
    }

    return $value === NULL || $value === '';
  }

  /**
   * The method defines more details about properties of the citeref fields.
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['citeref_type'] = DataDefinition::create('string')
      ->setLabel(t('Type'))
      ->setDescription(t('The ID of the citation type.'));
    $properties['citeref_style'] = DataDefinition::create('string')
      ->setLabel(t('Style'))
      ->setDescription(t('The style of the citation style language.'));
    $properties['citeref_id'] = DataDefinition::create('string')
      ->setLabel(t('ID'));
    $properties['citeref_record'] = DataDefinition::create('string')
      ->setLabel(t('Citation'));

    return $properties;
  }

  /**
   * Stores the values for the citeref fields into the database.
   *
   * @return array[]
   *   Returns an array of schema API columns.
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {

    $columns = [
      'citeref_type' => [
        'type' => 'varchar',
        'length' => 10,
        'description' => 'Type of the citation such as DOI, Handle, ARK, Url, Urn, and Other.',
      ],
      'citeref_style' => [
        'type' => 'varchar',
        'length' => 255,
        'description' => 'The style of the citation style language.',
      ],
      'citeref_id' => [
        'type' => 'varchar',
        'length' => 255,
        'description' => 'The citation identifier',
      ],
      'citeref_record' => [
        'type' => 'varchar',
        'length' => 2048,
        'description' => 'The citation record',
      ],
    ];

    return [
      'columns' => $columns,
    ];
  }

}
