<?php

namespace Drupal\citeref_field\ProxyClass;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Extension\ModuleUninstallValidatorInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Module uninstall validator proxy service class.
 * An citation or reference field module uninstall validator enables the lazy load for the service container.
 */
class CiterefFieldUninstallValidator implements ModuleUninstallValidatorInterface
{
    use DependencySerializationTrait;

    /**
     * The proxy service.
     *
     * @var string
     */
    protected $drupalProxyService;

    /**
     * The lazy loaded proxy service.
     *
     * @var \Drupal\citeref_field\CiterefFieldUninstallValidator
     */
    protected $service;

    /**
     * The service container.
     *
     * @var ContainerInterface
     */
    protected $container;

    /**
     * Constructs a ProxyClass Drupal proxy object.
     *
     * The container.
     *
     * @param  string  $drupal_proxy_service
     */
    public function __construct(ContainerInterface $container, $drupal_proxy_service)
    {
        $this->container = $container;
        $this->drupalProxyService = $drupal_proxy_service;
    }

    /**
     * Lazy loads the service from the container.
     *
     * Returns the constructed service.
     *
     * @return \Drupal\citeref_field\CiterefFieldUninstallValidator|object|null
     */
    protected function lazyLoad()
    {
        if (! isset($this->service)) {
            $this->service = $this->container->get($this->drupalProxyService);
        }

        return $this->service;
    }

    /**
     * Load "validate" function from the service container
     *
     * @param  string  $module
     * @return array|string[]
     *
     * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
     * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
     */
    public function validate($module)
    {
        return $this->lazyLoad()->validate($module);
    }

    /**
     * Sets the string translation service to use.
     *
     * @return mixed
     */
    public function setStringTranslation(TranslationInterface $translation)
    {
        return $this->lazyLoad()->setStringTranslation($translation);
    }
}
